@@ -7,7 +7,7 @@ from fontTools.misc.fixedTools import otRound
 
 from ufo2ft.featureWriters import BaseFeatureWriter, ast
 from ufo2ft.fontInfoData import getAttrWithFallback
-from ufo2ft.util import classifyGlyphs, unicodeInScripts
+from ufo2ft.util import classifyGlyphs, quantize, unicodeInScripts
 
 
 class AbstractMarkPos:
@@ -260,8 +260,13 @@ class MarkFeatureWriter(BaseFeatureWriter):
     names (see below). If Indic glyphs contain anchors with names not in those
     lists, the anchors' vertical position relative to the half of the UPEM
     square is used to decide whether they are considered above or below.
+
+    If the `quantization` argument is given in the filter options, the resulting
+    anchors are rounded to the nearest multiple of the quantization value.
     """
 
+    options = dict(quantization=1)
+
     tableTag = "GPOS"
     features = frozenset(["mark", "mkmk", "abvm", "blwm"])
 
@@ -296,7 +301,7 @@ class MarkFeatureWriter(BaseFeatureWriter):
 
     def setContext(self, font, feaFile, compiler=None):
         ctx = super().setContext(font, feaFile, compiler=compiler)
-        ctx.gdefClasses = ast.getGDEFGlyphClasses(feaFile)
+        ctx.gdefClasses = self.getGDEFGlyphClasses()
         ctx.anchorLists = self._getAnchorLists()
         ctx.anchorPairs = self._getAnchorPairs()
 
@@ -330,7 +335,9 @@ class MarkFeatureWriter(BaseFeatureWriter):
                     self.log.warning(
                         "duplicate anchor '%s' in glyph '%s'", anchorName, glyphName
                     )
-                a = self.NamedAnchor(name=anchorName, x=anchor.x, y=anchor.y)
+                x = quantize(anchor.x, self.options.quantization)
+                y = quantize(anchor.y, self.options.quantization)
+                a = self.NamedAnchor(name=anchorName, x=x, y=y)
                 anchorDict[anchorName] = a
             if anchorDict:
                 result[glyphName] = list(anchorDict.values())
@@ -852,7 +859,7 @@ class MarkFeatureWriter(BaseFeatureWriter):
     def _getIndicGlyphs(self):
         cmap = self.makeUnicodeToGlyphNameMapping()
         unicodeIsIndic = partial(unicodeInScripts, scripts=self.indicScripts)
-        if any(unicodeIsIndic for uv in cmap):
+        if any(unicodeIsIndic(uv) for uv in cmap):
             # If there are any characters from Indic scripts in the cmap, we
             # compile a temporary GSUB table to resolve substitutions and get
             # the set of all the "Indic" glyphs, including alternate glyphs.
@@ -875,9 +882,11 @@ class MarkFeatureWriter(BaseFeatureWriter):
             return False
 
         feaFile = self.context.feaFile
-        feaFile.statements.extend(newClassDefs)
-        # add empty line to separate classes from following statements
-        feaFile.statements.append(ast.Comment(""))
-        for _, feature in sorted(features.items()):
-            feaFile.statements.append(feature)
+
+        self._insert(
+            feaFile=feaFile,
+            markClassDefs=newClassDefs,
+            features=[features[tag] for tag in sorted(features.keys())],
+        )
+
         return True
