@@ -18,13 +18,19 @@ from fontTools.misc.fixedTools import otRound
 from fontTools.pens.boundsPen import ControlBoundsPen
 from fontTools.pens.reverseContourPen import ReverseContourPen
 from fontTools.pens.t2CharStringPen import T2CharStringPen
-from fontTools.pens.ttGlyphPen import TTGlyphPen
+from fontTools.pens.ttGlyphPen import TTGlyphPointPen
 from fontTools.ttLib import TTFont, newTable
 from fontTools.ttLib.tables._g_l_y_f import USE_MY_METRICS, Glyph
 from fontTools.ttLib.tables._h_e_a_d import mac_epoch_diff
 from fontTools.ttLib.tables.O_S_2f_2 import Panose
 
-from ufo2ft.constants import COLOR_LAYERS_KEY, COLOR_PALETTES_KEY
+from ufo2ft.constants import (
+    COLOR_LAYERS_KEY,
+    COLOR_PALETTES_KEY,
+    COLR_CLIP_BOXES_KEY,
+    OPENTYPE_META_KEY,
+    UNICODE_VARIATION_SEQUENCES_KEY,
+)
 from ufo2ft.errors import InvalidFontData
 from ufo2ft.fontInfoData import (
     dateStringForNow,
@@ -82,6 +88,7 @@ class BaseOutlineCompiler:
             "vhea",
             "COLR",
             "CPAL",
+            "meta",
         ]
     )
 
@@ -131,6 +138,7 @@ class BaseOutlineCompiler:
         self.colorLayers = (
             COLOR_LAYERS_KEY in self.ufo.lib and COLOR_PALETTES_KEY in self.ufo.lib
         )
+        self.meta = OPENTYPE_META_KEY in self.ufo.lib
 
         # write the glyph order
         self.otf.setGlyphOrder(self.glyphOrder)
@@ -150,6 +158,8 @@ class BaseOutlineCompiler:
         if self.colorLayers:
             self.setupTable_COLR()
             self.setupTable_CPAL()
+        if self.meta:
+            self.setupTable_meta()
         self.setupOtherTables()
         self.importTTX()
 
@@ -504,6 +514,33 @@ class BaseOutlineCompiler:
             cmap12_3_10.cmap = nonBMP
             # update tables registry
             cmap.tables = [cmap4_0_3, cmap4_3_1, cmap12_0_4, cmap12_3_10]
+        # unicode variation sequences
+        uvsMapping = self.ufo.lib.get(UNICODE_VARIATION_SEQUENCES_KEY)
+        if uvsMapping:
+            from fontTools.ttLib.tables._c_m_a_p import cmap_format_14
+
+            cmap14_0_5 = cmap_format_14(14)
+            cmap14_0_5.platformID = 0
+            cmap14_0_5.platEncID = 5
+            cmap14_0_5.language = 0
+            cmap14_0_5.cmap = {}
+            if nonBMP:
+                mapping = nonBMP
+            uvsDict = dict()
+            # public.unicodeVariationSequences uses hex strings as keys and
+            # a dict of dicts, while cmap uses ints and a dict of tuples.
+            for hexvs, glyphMapping in uvsMapping.items():
+                uvsList = []
+                for hexvalue, glyphName in glyphMapping.items():
+                    value = int(hexvalue, 16)
+                    if glyphName == mapping[value]:
+                        uvsList.append((value, None))
+                    else:
+                        uvsList.append((value, glyphName))
+                uvsDict[int(hexvs, 16)] = uvsList
+            cmap14_0_5.uvsDict = uvsDict
+            # update tables registry
+            cmap.tables.append(cmap14_0_5)
 
     def setupTable_OS2(self):
         """
@@ -537,7 +574,7 @@ class BaseOutlineCompiler:
         # subscript, superscript, strikeout values, taken from AFDKO:
         # FDK/Tools/Programs/makeotf/makeotf_lib/source/hotconv/hot.c
         unitsPerEm = getAttrWithFallback(font.info, "unitsPerEm")
-        italicAngle = getAttrWithFallback(font.info, "italicAngle")
+        italicAngle = float(getAttrWithFallback(font.info, "italicAngle"))
         xHeight = getAttrWithFallback(font.info, "xHeight")
 
         def adjustOffset(offset, angle):
@@ -889,7 +926,7 @@ class BaseOutlineCompiler:
         font = self.ufo
         post.formatType = 3.0
         # italic angle
-        italicAngle = getAttrWithFallback(font.info, "italicAngle")
+        italicAngle = float(getAttrWithFallback(font.info, "italicAngle"))
         post.italicAngle = italicAngle
         # underline
         underlinePosition = getAttrWithFallback(
@@ -900,7 +937,9 @@ class BaseOutlineCompiler:
             font.info, "postscriptUnderlineThickness"
         )
         post.underlineThickness = otRound(underlineThickness)
-        post.isFixedPitch = getAttrWithFallback(font.info, "postscriptIsFixedPitch")
+        post.isFixedPitch = int(
+            getAttrWithFallback(font.info, "postscriptIsFixedPitch")
+        )
         # misc
         post.minMemType42 = 0
         post.maxMemType42 = 0
@@ -921,7 +960,18 @@ class BaseOutlineCompiler:
         layerInfo = self.ufo.lib[COLOR_LAYERS_KEY]
         glyphMap = self.otf.getReverseGlyphMap()
         if layerInfo:
-            self.otf["COLR"] = buildCOLR(layerInfo, glyphMap=glyphMap)
+            # unpack (glyphs, clipBox) tuples to a flat dict keyed by glyph name,
+            # as colorLib buildCOLR expects
+            clipBoxes = {
+                glyphName: tuple(box)
+                for glyphs, box in self.ufo.lib.get(COLR_CLIP_BOXES_KEY, ())
+                for glyphName in glyphs
+            }
+            self.otf["COLR"] = buildCOLR(
+                layerInfo,
+                glyphMap=glyphMap,
+                clipBoxes=clipBoxes,
+            )
 
     def setupTable_CPAL(self):
         """
@@ -945,6 +995,45 @@ class BaseOutlineCompiler:
         except ColorLibError as e:
             raise InvalidFontData("Failed to build CPAL table") from e
 
+    def setupTable_meta(self):
+        """
+        Make the meta table.
+
+        ***This should not be called externally.** Sublcasses
+        may override or supplement this method to handle the
+        table creation in a different way if desired.
+        """
+        if "meta" not in self.tables:
+            return
+
+        font = self.ufo
+        self.otf["meta"] = meta = newTable("meta")
+        ufo_meta = font.lib.get(OPENTYPE_META_KEY)
+        for key, value in ufo_meta.items():
+            if key in ["dlng", "slng"]:
+                if not isinstance(value, list) or not all(
+                    isinstance(string, str) for string in value
+                ):
+                    raise TypeError(
+                        f"public.openTypeMeta '{key}' value should "
+                        "be a list of strings"
+                    )
+                meta.data[key] = ",".join(value)
+            elif key in ["appl", "bild"]:
+                if not isinstance(value, bytes):
+                    raise TypeError(
+                        f"public.openTypeMeta '{key}' value should be bytes."
+                    )
+                meta.data[key] = value
+            elif isinstance(value, bytes):
+                meta.data[key] = value
+            elif isinstance(value, str):
+                meta.data[key] = value.encode("utf-8")
+            else:
+                raise TypeError(
+                    f"public.openTypeMeta '{key}' value should be bytes or a string."
+                )
+
     def setupOtherTables(self):
         """
         Make the other tables. The default implementation does nothing.
@@ -1140,6 +1229,9 @@ class OutlineOTFCompiler(BaseOutlineCompiler):
 
         self.otf["CFF "] = cff = newTable("CFF ")
         cff = cff.cff
+        # NOTE: Set up a back-reference to be used by some CFFFontSet methods
+        # down the line (as of fontTools 4.21.1).
+        cff.otFont = self.otf
         # set up the basics
         cff.major = 1
         cff.minor = 0
@@ -1215,8 +1307,8 @@ class OutlineOTFCompiler(BaseOutlineCompiler):
         )
         topDict.Weight = getAttrWithFallback(info, "postscriptWeightName")
         # populate various numbers
-        topDict.isFixedPitch = getAttrWithFallback(info, "postscriptIsFixedPitch")
-        topDict.ItalicAngle = getAttrWithFallback(info, "italicAngle")
+        topDict.isFixedPitch = int(getAttrWithFallback(info, "postscriptIsFixedPitch"))
+        topDict.ItalicAngle = float(getAttrWithFallback(info, "italicAngle"))
         underlinePosition = getAttrWithFallback(info, "postscriptUnderlinePosition")
         topDict.UnderlinePosition = otRound(underlinePosition)
         underlineThickness = getAttrWithFallback(info, "postscriptUnderlineThickness")
@@ -1304,9 +1396,9 @@ class OutlineTTFCompiler(BaseOutlineCompiler):
         ttGlyphs = {}
         for name in self.glyphOrder:
             glyph = allGlyphs[name]
-            pen = TTGlyphPen(allGlyphs)
+            pen = TTGlyphPointPen(allGlyphs)
             try:
-                glyph.draw(pen)
+                glyph.drawPoints(pen)
             except NotImplementedError:
                 logger.error("%r has invalid curve format; skipped", name)
                 ttGlyph = Glyph()
@@ -1457,6 +1549,9 @@ class StubGlyph:
     def draw(self, pen):
         pass
 
+    def drawPoints(self, pen):
+        pass
+
     def _drawDefaultNotdef(self, pen):
         # Draw contour in PostScript direction (counter-clockwise) by default. Reverse
         # for TrueType.
