@@ -1,9 +1,12 @@
+import importlib
 import logging
+import re
 from copy import deepcopy
 from inspect import getfullargspec
 
 from fontTools import subset, ttLib, unicodedata
 from fontTools.feaLib.builder import addOpenTypeFeatures
+from fontTools.misc.fixedTools import otRound
 from fontTools.misc.transform import Identity, Transform
 from fontTools.pens.reverseContourPen import ReverseContourPen
 from fontTools.pens.transformPen import TransformPen
@@ -226,6 +229,26 @@ def compileGSUB(featureFile, glyphOrder):
     return font.get("GSUB")
 
 
+def compileGDEF(featureFile, glyphOrder):
+    """Compile and return a GDEF table from `featureFile` (feaLib FeatureFile),
+    using the given `glyphOrder` (list of glyph names).
+    """
+    from fontTools.feaLib.ast import TableBlock
+
+    font = ttLib.TTFont()
+    font.setGlyphOrder(glyphOrder)
+    gdefDefined = False
+    for statement in featureFile.statements:
+        if isinstance(statement, TableBlock) and statement.name == "GDEF":
+            gdefDefined = True
+
+    if not gdefDefined:
+        addOpenTypeFeatures(font, featureFile, tables={"GDEF", "GPOS", "GSUB"})
+    else:
+        addOpenTypeFeatures(font, featureFile, tables={"GDEF"})
+    return font.get("GDEF")
+
+
 def closeGlyphsOverGSUB(gsub, glyphs):
     """Use the FontTools subsetter to perform a closure over the GSUB table
     given the initial `glyphs` (set of glyph names, str). Update the set
@@ -419,3 +442,48 @@ def _getDefaultNotdefGlyph(designSpaceDoc):
         except KeyError:
             notdefGlyph = None
     return notdefGlyph
+
+
+# NOTE about the security risk involved in using eval: the function below is
+# meant to be used to parse string coming from the command-line, which is
+# inherently "trusted"; if that weren't the case, a potential attacker
+# could do worse things than segfaulting the Python interpreter...
+
+
+def _kwargsEval(s):
+    return eval(
+        "dict(%s)" % s, {"__builtins__": {"True": True, "False": False, "dict": dict}}
+    )
+
+
+_pluginSpecRE = re.compile(
+    r"(?:([\w\.]+)::)?"  # MODULE_NAME + '::'
+    r"(\w+)"  # CLASS_NAME [required]
+    r"(?:\((.*)\))?"  # (KWARGS)
+)
+
+
+def _loadPluginFromString(spec, moduleName, isValidFunc):
+    spec = spec.strip()
+    m = _pluginSpecRE.match(spec)
+    if not m or (m.end() - m.start()) != len(spec):
+        raise ValueError(spec)
+    moduleName = m.group(1) or moduleName
+    className = m.group(2)
+    kwargs = m.group(3)
+
+    module = importlib.import_module(moduleName)
+    klass = getattr(module, className)
+    if not isValidFunc(klass):
+        raise TypeError(klass)
+    try:
+        options = _kwargsEval(kwargs) if kwargs else {}
+    except SyntaxError:
+        raise ValueError("options have incorrect format: %r" % kwargs)
+
+    return klass(**options)
+
+
+def quantize(number, factor):
+    """Round to a multiple of the given parameter"""
+    return factor * otRound(number / factor)
