@@ -1,9 +1,11 @@
 import logging
 import os
+import re
 from textwrap import dedent
 
 import pytest
 
+from ufo2ft.errors import InvalidFeaturesData
 from ufo2ft.featureCompiler import parseLayoutFeatures
 from ufo2ft.featureWriters import ast
 from ufo2ft.featureWriters.markFeatureWriter import (
@@ -179,7 +181,7 @@ class MarkFeatureWriterTest(FeatureWriterTest):
         ]
         generated = self.writeFeatures(testufo)
 
-        assert "ligComponent <anchor NULL>" in str(generated)
+        assert re.search(r"ligComponent\s+<anchor NULL>", str(generated))
 
     def test_skip_existing_feature(self, testufo):
         testufo.features.text = dedent(
@@ -187,7 +189,8 @@ class MarkFeatureWriterTest(FeatureWriterTest):
             markClass acutecomb <anchor 100 200> @MC_top;
             feature mark {
                 lookup mark1 {
-                    pos base a <anchor 100 200> mark @MC_top;
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
                 } mark1;
             } mark;
             """
@@ -204,7 +207,8 @@ class MarkFeatureWriterTest(FeatureWriterTest):
                 lookup mark2mark_top {
                     @MFS_mark2mark_top = [acutecomb tildecomb];
                     lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
-                    pos mark tildecomb <anchor 100 300> mark @MC_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
                 } mark2mark_top;
 
             } mkmk;
@@ -217,7 +221,8 @@ class MarkFeatureWriterTest(FeatureWriterTest):
             markClass acutecomb <anchor 100 200> @MC_top;
             feature mark {
                 lookup mark1 {
-                    pos base a <anchor 100 200> mark @MC_top;
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
                 } mark1;
             } mark;
             """
@@ -231,12 +236,15 @@ class MarkFeatureWriterTest(FeatureWriterTest):
 
             feature mark {
                 lookup mark2base {
-                    pos base a <anchor 100 200> mark @MC_top;
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
                 } mark2base;
 
                 lookup mark2liga {
-                    pos ligature f_i <anchor 100 500> mark @MC_top
-                        ligComponent <anchor 600 500> mark @MC_top;
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
                 } mark2liga;
 
             } mark;
@@ -245,13 +253,362 @@ class MarkFeatureWriterTest(FeatureWriterTest):
                 lookup mark2mark_top {
                     @MFS_mark2mark_top = [acutecomb tildecomb];
                     lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
-                    pos mark tildecomb <anchor 100 300> mark @MC_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
                 } mark2mark_top;
 
             } mkmk;
             """
         )
 
+    def test_insert_comment_before(self, testufo):
+        writer = MarkFeatureWriter()
+        testufo.features.text = dedent(
+            """\
+            markClass acutecomb <anchor 100 200> @MC_top;
+            feature mark {
+                #
+                # Automatic Code
+                #
+                lookup mark1 {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark1;
+            } mark;
+            """
+        )
+        feaFile = parseLayoutFeatures(testufo)
+
+        assert writer.write(testufo, feaFile)
+
+        assert str(feaFile) == dedent(
+            """\
+            markClass acutecomb <anchor 100 200> @MC_top;
+            markClass tildecomb <anchor 100 200> @MC_top;
+
+            feature mark {
+                lookup mark2base {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark2base;
+
+                lookup mark2liga {
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
+                } mark2liga;
+
+            } mark;
+
+            feature mark {
+                #
+                #
+                lookup mark1 {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark1;
+
+            } mark;
+
+            feature mkmk {
+                lookup mark2mark_top {
+                    @MFS_mark2mark_top = [acutecomb tildecomb];
+                    lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
+                } mark2mark_top;
+
+            } mkmk;
+            """
+        )
+
+        # test append mode ignores insert marker
+        generated = self.writeFeatures(testufo, mode="append")
+        assert str(generated) == dedent(
+            """\
+            markClass tildecomb <anchor 100 200> @MC_top;
+
+            feature mark {
+                lookup mark2base {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark2base;
+
+                lookup mark2liga {
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
+                } mark2liga;
+
+            } mark;
+
+            feature mkmk {
+                lookup mark2mark_top {
+                    @MFS_mark2mark_top = [acutecomb tildecomb];
+                    lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
+                } mark2mark_top;
+
+            } mkmk;
+            """
+        )
+
+    def test_insert_comment_after(self, testufo):
+        writer = MarkFeatureWriter()
+        testufo.features.text = dedent(
+            """\
+            markClass acutecomb <anchor 100 200> @MC_top;
+            feature mark {
+                lookup mark1 {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark1;
+                #
+                # Automatic Code
+                #
+            } mark;
+            """
+        )
+        feaFile = parseLayoutFeatures(testufo)
+
+        assert writer.write(testufo, feaFile)
+
+        assert str(feaFile) == dedent(
+            """\
+            markClass acutecomb <anchor 100 200> @MC_top;
+            feature mark {
+                lookup mark1 {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark1;
+
+                #
+                #
+            } mark;
+
+            markClass tildecomb <anchor 100 200> @MC_top;
+
+            feature mark {
+                lookup mark2base {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark2base;
+
+                lookup mark2liga {
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
+                } mark2liga;
+
+            } mark;
+
+            feature mkmk {
+                lookup mark2mark_top {
+                    @MFS_mark2mark_top = [acutecomb tildecomb];
+                    lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
+                } mark2mark_top;
+
+            } mkmk;
+            """
+        )
+
+        # test append mode ignores insert marker
+        generated = self.writeFeatures(testufo, mode="append")
+        assert str(generated) == dedent(
+            """\
+            markClass tildecomb <anchor 100 200> @MC_top;
+
+            feature mark {
+                lookup mark2base {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark2base;
+
+                lookup mark2liga {
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
+                } mark2liga;
+
+            } mark;
+
+            feature mkmk {
+                lookup mark2mark_top {
+                    @MFS_mark2mark_top = [acutecomb tildecomb];
+                    lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
+                } mark2mark_top;
+
+            } mkmk;
+            """
+        )
+
+    def test_insert_comment_middle(self, testufo):
+        writer = MarkFeatureWriter()
+        testufo.features.text = dedent(
+            """\
+            markClass acutecomb <anchor 100 200> @MC_top;
+            feature mark {
+                lookup mark1 {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark1;
+                #
+                # Automatic Code
+                #
+                lookup mark2 {
+                    pos base a
+                        <anchor 150 250> mark @MC_top;
+                } mark2;
+            } mark;
+            """
+        )
+        feaFile = parseLayoutFeatures(testufo)
+
+        with pytest.raises(
+            InvalidFeaturesData,
+            match="Insert marker has rules before and after, feature mark "
+            "cannot be inserted.",
+        ):
+            writer.write(testufo, feaFile)
+
+        # test append mode ignores insert marker
+        generated = self.writeFeatures(testufo, mode="append")
+        assert str(generated) == dedent(
+            """\
+            markClass tildecomb <anchor 100 200> @MC_top;
+
+            feature mark {
+                lookup mark2base {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark2base;
+
+                lookup mark2liga {
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
+                } mark2liga;
+
+            } mark;
+
+            feature mkmk {
+                lookup mark2mark_top {
+                    @MFS_mark2mark_top = [acutecomb tildecomb];
+                    lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
+                } mark2mark_top;
+
+            } mkmk;
+            """
+        )
+
+    def test_insert_comment_outside_block(self, testufo):
+        writer = MarkFeatureWriter()
+        testufo.features.text = dedent(
+            """\
+            #
+            # Automatic Code
+            #
+            """
+        )
+        feaFile = parseLayoutFeatures(testufo)
+
+        assert writer.write(testufo, feaFile)
+
+        testufo.features.text = dedent(
+            """\
+            #
+            # Automatic Code
+            #
+            markClass acutecomb <anchor 100 200> @MC_top;
+            feature mark {
+                lookup mark1 {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark1;
+
+            } mark;
+            """
+        )
+        feaFile = parseLayoutFeatures(testufo)
+
+        assert writer.write(testufo, feaFile)
+
+        # test append mode
+        writer = MarkFeatureWriter(mode="append")
+        assert writer.write(testufo, feaFile)
+
+    def test_defs_and_lookups_first(self, testufo):
+        testufo.newGlyph("circumflexcomb")
+        writer = MarkFeatureWriter()
+        testufo.features.text = dedent(
+            """\
+            feature mkmk {
+                # Automatic Code
+                # Move acutecomb down and right if preceded by circumflexcomb
+                lookup move_acutecomb {
+                    lookupflag UseMarkFilteringSet [acutecomb circumflexcomb];
+                    pos circumflexcomb acutecomb' <0 20 0 20>;
+                } move_acutecomb;
+            } mkmk;
+            """
+        )
+        feaFile = parseLayoutFeatures(testufo)
+
+        assert writer.write(testufo, feaFile)
+
+        assert str(feaFile) == dedent(
+            """\
+            markClass acutecomb <anchor 100 200> @MC_top;
+            markClass tildecomb <anchor 100 200> @MC_top;
+
+            feature mark {
+                lookup mark2base {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark2base;
+
+                lookup mark2liga {
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
+                } mark2liga;
+
+            } mark;
+
+            feature mkmk {
+                lookup mark2mark_top {
+                    @MFS_mark2mark_top = [acutecomb tildecomb];
+                    lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
+                } mark2mark_top;
+
+            } mkmk;
+
+            feature mkmk {
+                # Move acutecomb down and right if preceded by circumflexcomb
+                lookup move_acutecomb {
+                    lookupflag UseMarkFilteringSet [acutecomb circumflexcomb];
+                    pos circumflexcomb acutecomb' <0 20 0 20>;
+                } move_acutecomb;
+
+            } mkmk;
+            """
+        )
+
     def test_mark_mkmk_features(self, testufo):
         writer = MarkFeatureWriter()  # by default both mark + mkmk are built
         feaFile = ast.FeatureFile()
@@ -264,12 +621,15 @@ class MarkFeatureWriterTest(FeatureWriterTest):
 
             feature mark {
                 lookup mark2base {
-                    pos base a <anchor 100 200> mark @MC_top;
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
                 } mark2base;
 
                 lookup mark2liga {
-                    pos ligature f_i <anchor 100 500> mark @MC_top
-                        ligComponent <anchor 600 500> mark @MC_top;
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
                 } mark2liga;
 
             } mark;
@@ -278,7 +638,8 @@ class MarkFeatureWriterTest(FeatureWriterTest):
                 lookup mark2mark_top {
                     @MFS_mark2mark_top = [acutecomb tildecomb];
                     lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
-                    pos mark tildecomb <anchor 100 300> mark @MC_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
                 } mark2mark_top;
 
             } mkmk;
@@ -347,12 +708,16 @@ class MarkFeatureWriterTest(FeatureWriterTest):
 
             feature mark {
                 lookup mark2base {
-                    pos base a <anchor 250 -100> mark @MC_markbottom <anchor 250 600> mark @MC_marktop;
+                    pos base a
+                        <anchor 250 -100> mark @MC_markbottom
+                        <anchor 250 600> mark @MC_marktop;
                 } mark2base;
 
                 lookup mark2liga {
-                    pos ligature f_i <anchor 200 700> mark @MC_marktop
-                        ligComponent <anchor 500 700> mark @MC_marktop;
+                    pos ligature f_i
+                            <anchor 200 700> mark @MC_marktop
+                        ligComponent
+                            <anchor 500 700> mark @MC_marktop;
                 } mark2liga;
 
             } mark;
@@ -361,13 +726,15 @@ class MarkFeatureWriterTest(FeatureWriterTest):
                 lookup mark2mark_bottom {
                     @MFS_mark2mark_bottom = [cedillacomb gravecomb];
                     lookupflag UseMarkFilteringSet @MFS_mark2mark_bottom;
-                    pos mark gravecomb <anchor 150 600> mark @MC_markbottom;
+                    pos mark gravecomb
+                        <anchor 150 600> mark @MC_markbottom;
                 } mark2mark_bottom;
 
                 lookup mark2mark_top {
                     @MFS_mark2mark_top = [gravecomb];
                     lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
-                    pos mark gravecomb <anchor 150 800> mark @MC_marktop;
+                    pos mark gravecomb
+                        <anchor 150 800> mark @MC_marktop;
                 } mark2mark_top;
 
             } mkmk;
@@ -428,22 +795,29 @@ class MarkFeatureWriterTest(FeatureWriterTest):
 
             feature abvm {
                 lookup abvm_mark2base {
-                    pos base ka-kannada.base <anchor 291 547> mark @MC_top <anchor 391 460> mark @MC_topright;
+                    pos base ka-kannada.base
+                        <anchor 291 547> mark @MC_top
+                        <anchor 391 460> mark @MC_topright;
                 } abvm_mark2base;
 
             } abvm;
 
             feature blwm {
                 lookup blwm_mark2base {
-                    pos base ka-kannada <anchor 290 0> mark @MC_bottom;
-                    pos base ka-kannada.base <anchor 290 0> mark @MC_bottom;
+                    pos base ka-kannada
+                        <anchor 290 0> mark @MC_bottom;
+                    pos base ka-kannada.base
+                        <anchor 290 0> mark @MC_bottom;
                 } blwm_mark2base;
 
             } blwm;
 
             feature mark {
                 lookup mark2base {
-                    pos base dottedCircle <anchor 297 0> mark @MC_bottom <anchor 297 552> mark @MC_top <anchor 491 458> mark @MC_topright;
+                    pos base dottedCircle
+                        <anchor 297 0> mark @MC_bottom
+                        <anchor 297 552> mark @MC_top
+                        <anchor 491 458> mark @MC_topright;
                 } mark2base;
 
             } mark;
@@ -528,52 +902,70 @@ class MarkFeatureWriterTest(FeatureWriterTest):
 
             feature abvm {
                 lookup abvm_mark2liga {
-                    pos ligature foo_bar_baz <anchor 100 500> mark @MC_top
-                        ligComponent <anchor NULL>
-                        ligComponent <anchor 1000 500> mark @MC_top;
-                    pos ligature bar_foo <anchor NULL>
-                        ligComponent <anchor 600 501> mark @MC_top;
+                    pos ligature foo_bar_baz
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor NULL>
+                        ligComponent
+                            <anchor 1000 500> mark @MC_top;
+                    pos ligature bar_foo
+                            <anchor NULL>
+                        ligComponent
+                            <anchor 600 501> mark @MC_top;
                 } abvm_mark2liga;
 
                 lookup abvm_mark2mark_top {
                     @MFS_abvm_mark2mark_top = [foocomb];
                     lookupflag UseMarkFilteringSet @MFS_abvm_mark2mark_top;
-                    pos mark foocomb <anchor 100 190> mark @MC_top;
+                    pos mark foocomb
+                        <anchor 100 190> mark @MC_top;
                 } abvm_mark2mark_top;
 
             } abvm;
 
             feature blwm {
                 lookup blwm_mark2liga {
-                    pos ligature foo_bar_baz <anchor 100 10> mark @MC_bottom
-                        ligComponent <anchor NULL>
-                        ligComponent <anchor 1100 499> mark @MC_bar;
+                    pos ligature foo_bar_baz
+                            <anchor 100 10> mark @MC_bottom
+                        ligComponent
+                            <anchor NULL>
+                        ligComponent
+                            <anchor 1100 499> mark @MC_bar;
                 } blwm_mark2liga;
 
                 lookup blwm_mark2mark_bar {
                     @MFS_blwm_mark2mark_bar = [barcomb];
                     lookupflag UseMarkFilteringSet @MFS_blwm_mark2mark_bar;
-                    pos mark barcomb <anchor 100 440> mark @MC_bar;
+                    pos mark barcomb
+                        <anchor 100 440> mark @MC_bar;
                 } blwm_mark2mark_bar;
 
                 lookup blwm_mark2mark_bottom {
                     @MFS_blwm_mark2mark_bottom = [bazcomb];
                     lookupflag UseMarkFilteringSet @MFS_blwm_mark2mark_bottom;
-                    pos mark bazcomb <anchor 100 -34> mark @MC_bottom;
+                    pos mark bazcomb
+                        <anchor 100 -34> mark @MC_bottom;
                 } blwm_mark2mark_bottom;
 
             } blwm;
 
             feature mark {
                 lookup mark2base {
-                    pos base a <anchor 100 200> mark @MC_top;
-                    pos base c <anchor 240 0> mark @MC_bottom;
-                    pos base dottedCircle <anchor 491 458> mark @MC_bar <anchor 297 0> mark @MC_bottom <anchor 297 552> mark @MC_top;
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                    pos base c
+                        <anchor 240 0> mark @MC_bottom;
+                    pos base dottedCircle
+                        <anchor 491 458> mark @MC_bar
+                        <anchor 297 0> mark @MC_bottom
+                        <anchor 297 552> mark @MC_top;
                 } mark2base;
 
                 lookup mark2liga {
-                    pos ligature f_i <anchor 100 500> mark @MC_top
-                        ligComponent <anchor 600 500> mark @MC_top;
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
                 } mark2liga;
 
             } mark;
@@ -582,13 +974,15 @@ class MarkFeatureWriterTest(FeatureWriterTest):
                 lookup mark2mark_bottom {
                     @MFS_mark2mark_bottom = [cedillacomb];
                     lookupflag UseMarkFilteringSet @MFS_mark2mark_bottom;
-                    pos mark cedillacomb <anchor 20 -309> mark @MC_bottom;
+                    pos mark cedillacomb
+                        <anchor 20 -309> mark @MC_bottom;
                 } mark2mark_bottom;
 
                 lookup mark2mark_top {
                     @MFS_mark2mark_top = [acutecomb tildecomb];
                     lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
-                    pos mark tildecomb <anchor 100 300> mark @MC_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
                 } mark2mark_top;
 
             } mkmk;
@@ -645,17 +1039,22 @@ class MarkFeatureWriterTest(FeatureWriterTest):
 
             feature mark {
                 lookup mark2base {
-                    pos base D <anchor 320 360> mark @MC_center;
+                    pos base D
+                        <anchor 320 360> mark @MC_center;
                 } mark2base;
 
                 lookup mark2base_1 {
-                    pos base D <anchor 300 700> mark @MC_top;
-                    pos base a <anchor 100 200> mark @MC_top;
+                    pos base D
+                        <anchor 300 700> mark @MC_top;
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
                 } mark2base_1;
 
                 lookup mark2liga {
-                    pos ligature f_i <anchor 100 500> mark @MC_top
-                        ligComponent <anchor 600 500> mark @MC_top;
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
                 } mark2liga;
 
             } mark;
@@ -664,8 +1063,91 @@ class MarkFeatureWriterTest(FeatureWriterTest):
                 lookup mark2mark_top {
                     @MFS_mark2mark_top = [acutecomb dotaccentcomb tildecomb];
                     lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
-                    pos mark dotaccentcomb <anchor 0 300> mark @MC_top;
-                    pos mark tildecomb <anchor 100 300> mark @MC_top;
+                    pos mark dotaccentcomb
+                        <anchor 0 300> mark @MC_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
+                } mark2mark_top;
+
+            } mkmk;
+            """
+        )
+
+    def test_mark_mkmk_features_with_GDEF_and_openTypeCategories(self, testufo):
+        # this glyph has compatible anchors and has an openTypeCategories "base"
+        # value
+        D = testufo.newGlyph("D")
+        D.anchors = [
+            {"name": "top", "x": 300, "y": 700},
+            {"name": "center", "x": 320, "y": 360},
+        ]
+        # these glyphs have compatible anchors but since they not listed in
+        # the GDEF groups, they won't be included in the mark/mkmk feature
+        testufo.newGlyph("Alpha").appendAnchor({"name": "topleft", "x": -10, "y": 400})
+        testufo.newGlyph("psili").appendAnchor({"name": "_topleft", "x": 0, "y": 50})
+        dotaccentcomb = testufo.newGlyph("dotaccentcomb")
+        # this mark glyph has more than one mark anchor, and both will be
+        # generated. Since the two mark anchors cannot cohabit in the same
+        # mark lookup, two lookups will be generated.
+        dotaccentcomb.anchors = [
+            {"name": "_center", "x": 0, "y": 0},
+            {"name": "_top", "x": 0, "y": 0},
+            {"name": "top", "x": 0, "y": 300},
+        ]
+        # will be ignored because in GDEF table below
+        testufo.lib["public.openTypeCategories"] = {
+            "D": "base",
+            "dotaccentcomb": "mark",
+            "tildecomb": "base",
+        }
+        testufo.features.text = dedent(
+            """\
+            @Bases = [a];
+            @Marks = [acutecomb tildecomb];
+            table GDEF {
+                GlyphClassDef @Bases, [f_i], @Marks, ;
+            } GDEF;
+            """
+        )
+        testufo.glyphOrder = [
+            "Alpha",
+            "D",
+            "a",
+            "acutecomb",
+            "dotaccentcomb",
+            "f_i",
+            "psili",
+            "tildecomb",
+        ]
+
+        generated = self.writeFeatures(testufo)
+
+        assert str(generated) == dedent(
+            """\
+            markClass acutecomb <anchor 100 200> @MC_top;
+            markClass tildecomb <anchor 100 200> @MC_top;
+
+            feature mark {
+                lookup mark2base {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark2base;
+
+                lookup mark2liga {
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
+                } mark2liga;
+
+            } mark;
+
+            feature mkmk {
+                lookup mark2mark_top {
+                    @MFS_mark2mark_top = [acutecomb tildecomb];
+                    lookupflag UseMarkFilteringSet @MFS_mark2mark_top;
+                    pos mark tildecomb
+                        <anchor 100 300> mark @MC_top;
                 } mark2mark_top;
 
             } mkmk;
@@ -685,11 +1167,13 @@ class MarkFeatureWriterTest(FeatureWriterTest):
 
             feature mark {
                 lookup mark2base {
-                    pos base a <anchor 515 581> mark @MC_topA;
+                    pos base a
+                        <anchor 515 581> mark @MC_topA;
                 } mark2base;
 
                 lookup mark2base_1 {
-                    pos base e <anchor -21 396> mark @MC_topE;
+                    pos base e
+                        <anchor -21 396> mark @MC_topE;
                 } mark2base_1;
 
             } mark;
@@ -721,17 +1205,25 @@ class MarkFeatureWriterTest(FeatureWriterTest):
 
             feature mark {
                 lookup mark2liga {
-                    pos ligature f_f <anchor 101 501> mark @MC_topOther
-                        ligComponent <anchor 601 501> mark @MC_topOther;
-                    pos ligature f_l <anchor NULL>
-                        ligComponent <anchor 602 502> mark @MC_topOther;
+                    pos ligature f_f
+                            <anchor 101 501> mark @MC_topOther
+                        ligComponent
+                            <anchor 601 501> mark @MC_topOther;
+                    pos ligature f_l
+                            <anchor NULL>
+                        ligComponent
+                            <anchor 602 502> mark @MC_topOther;
                 } mark2liga;
 
                 lookup mark2liga_1 {
-                    pos ligature f_i <anchor 100 500> mark @MC_top
-                        ligComponent <anchor 600 500> mark @MC_top;
-                    pos ligature f_l <anchor 102 502> mark @MC_top
-                        ligComponent <anchor NULL>;
+                    pos ligature f_i
+                            <anchor 100 500> mark @MC_top
+                        ligComponent
+                            <anchor 600 500> mark @MC_top;
+                    pos ligature f_l
+                            <anchor 102 502> mark @MC_top
+                        ligComponent
+                            <anchor NULL>;
                 } mark2liga_1;
 
             } mark;
@@ -773,11 +1265,13 @@ class MarkFeatureWriterTest(FeatureWriterTest):
 
             feature mark {
                 lookup mark2base {
-                    pos base a <anchor 150 550> mark @MC_topOther;
+                    pos base a
+                        <anchor 150 550> mark @MC_topOther;
                 } mark2base;
 
                 lookup mark2base_1 {
-                    pos base a <anchor 100 500> mark @MC_top;
+                    pos base a
+                        <anchor 100 500> mark @MC_top;
                 } mark2base_1;
 
             } mark;
@@ -796,7 +1290,30 @@ class MarkFeatureWriterTest(FeatureWriterTest):
 
             feature mark {
                 lookup mark2base {
-                    pos base a <anchor 100 200> mark @MC_top;
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
+                } mark2base;
+
+            } mark;
+            """
+        )
+
+    def test_quantize(self, testufo):
+        testufo.newGlyph("ogonekcomb").anchors = [
+            {"name": "_top", "x": 236, "y": 188},
+        ]
+        testufo.lib["public.skipExportGlyphs"] = ["f_i", "tildecomb"]
+        generated = self.writeFeatures(testufo, quantization=50)
+
+        assert str(generated) == dedent(
+            """\
+            markClass acutecomb <anchor 100 200> @MC_top;
+            markClass ogonekcomb <anchor 250 200> @MC_top;
+
+            feature mark {
+                lookup mark2base {
+                    pos base a
+                        <anchor 100 200> mark @MC_top;
                 } mark2base;
 
             } mark;
