@@ -4,6 +4,7 @@ import os
 import sys
 
 import pytest
+from fontTools.pens.boundsPen import BoundsPen
 
 from ufo2ft import (
     compileInterpolatableTTFs,
@@ -12,6 +13,8 @@ from ufo2ft import (
     compileVariableCFF2,
     compileVariableTTF,
 )
+from ufo2ft.constants import KEEP_GLYPH_NAMES
+from ufo2ft.filters import TransformationsFilter
 
 
 def getpath(filename):
@@ -166,8 +169,8 @@ class IntegrationTest:
         "subroutinizer, cff_version, expected_ttx",
         [
             (None, 1, "TestFont-CFF.ttx"),
-            ("compreffor", 1, "TestFont-CFF.ttx"),
-            ("cffsubr", 1, "TestFont-CFF-cffsubr.ttx"),
+            ("compreffor", 1, "TestFont-CFF-compreffor.ttx"),
+            ("cffsubr", 1, "TestFont-CFF.ttx"),
             (None, 2, "TestFont-CFF2-cffsubr.ttx"),
             # ("compreffor", 2, "TestFont-CFF2-compreffor.ttx"),
             ("cffsubr", 2, "TestFont-CFF2-cffsubr.ttx"),
@@ -229,13 +232,102 @@ class IntegrationTest:
         ],
     )
     def test_drop_glyph_names(self, testufo, output_format, options, expected_ttx):
-        from ufo2ft.constants import KEEP_GLYPH_NAMES
-
         testufo.lib[KEEP_GLYPH_NAMES] = False
         compile_func = globals()[f"compile{output_format}"]
         ttf = compile_func(testufo, **options)
         expectTTX(ttf, expected_ttx)
 
+    @pytest.mark.parametrize(
+        "output_format, options, expected_ttx",
+        [
+            ("VariableTTF", {}, "TestVariableFont-TTF-post3.ttx"),
+            ("VariableCFF2", {}, "TestVariableFont-CFF2-post3.ttx"),
+        ],
+    )
+    def test_drop_glyph_names_variable(
+        self, designspace, output_format, options, expected_ttx
+    ):
+        # set keepGlyphNames in the default UFO.lib where postProcessor finds it
+        designspace.findDefault().font.lib[KEEP_GLYPH_NAMES] = False
+        compile_func = globals()[f"compile{output_format}"]
+        ttf = compile_func(designspace, **options)
+        expectTTX(ttf, expected_ttx)
+
+    @pytest.mark.parametrize(
+        "compileFunc",
+        [
+            compileOTF,
+            compileTTF,
+        ],
+    )
+    def test_compile_filters(self, compileFunc, FontClass):
+        ufo = FontClass(getpath("LayerFont-Regular.ufo"))
+        filters = [TransformationsFilter(OffsetY=10)]
+        ttf = compileFunc(ufo, filters=filters)
+
+        pen1 = BoundsPen(ufo)
+        glyph = ufo["a"]
+        glyph.draw(pen1)
+
+        glyphSet = ttf.getGlyphSet()
+        tt_glyph = glyphSet["a"]
+        pen2 = BoundsPen(glyphSet)
+        tt_glyph.draw(pen2)
+
+        assert pen1.bounds[0] == pen2.bounds[0]
+        assert pen1.bounds[1] + 10 == pen2.bounds[1]
+        assert pen1.bounds[2] == pen2.bounds[2]
+        assert pen1.bounds[3] + 10 == pen2.bounds[3]
+
+    @pytest.mark.parametrize(
+        "compileFunc",
+        [
+            compileVariableTTF,
+            compileVariableCFF2,
+        ],
+    )
+    def test_compileVariable_filters(self, designspace, compileFunc):
+        filters = [TransformationsFilter(OffsetY=10)]
+        varfont = compileFunc(designspace, filters=filters)
+
+        ufo = designspace.sources[0].font
+        pen1 = BoundsPen(ufo)
+        glyph = ufo["a"]
+        glyph.draw(pen1)
+
+        glyphSet = varfont.getGlyphSet()
+        tt_glyph = glyphSet["a"]
+        pen2 = BoundsPen(glyphSet)
+        tt_glyph.draw(pen2)
+
+        assert pen1.bounds[0] == pen2.bounds[0]
+        assert pen1.bounds[1] + 10 == pen2.bounds[1]
+        assert pen1.bounds[2] == pen2.bounds[2]
+        assert pen1.bounds[3] + 10 == pen2.bounds[3]
+
+    def test_compileInterpolatableTTFs(self, FontClass):
+        ufos = [
+            FontClass(getpath("NestedComponents-Regular.ufo")),
+            FontClass(getpath("NestedComponents-Bold.ufo")),
+        ]
+        filters = [TransformationsFilter(OffsetY=10)]
+        ttfs = compileInterpolatableTTFs(ufos, filters=filters)
+
+        for i, ttf in enumerate(ttfs):
+            glyph = ufos[i]["a"]
+            pen1 = BoundsPen(ufos[i])
+            glyph.draw(pen1)
+
+            glyphSet = ttf.getGlyphSet()
+            tt_glyph = glyphSet["uni0061"]
+            pen2 = BoundsPen(glyphSet)
+            tt_glyph.draw(pen2)
+
+            assert pen1.bounds[0] == pen2.bounds[0]
+            assert pen1.bounds[1] + 10 == pen2.bounds[1]
+            assert pen1.bounds[2] == pen2.bounds[2]
+            assert pen1.bounds[3] + 10 == pen2.bounds[3]
+
 
 if __name__ == "__main__":
     sys.exit(pytest.main(sys.argv))
